@@ -1,8 +1,8 @@
-## haplotype.R (2023-11-21)
+## haplotype.R (2025-03-07)
 
 ##   Haplotype Extraction, Frequencies, and Networks
 
-## Copyright 2009-2023 Emmanuel Paradis, 2013 Klaus Schliep
+## Copyright 2009-2025 Emmanuel Paradis, 2013 Klaus Schliep
 
 ## This file is part of the R-package `pegas'.
 ## See the file ../DESCRIPTION for licensing issues.
@@ -30,9 +30,71 @@ getAllCombs <- function(n) {
     res
 }
 
-rmst <- function(d, B = NULL, stop.criterion = NULL, iter.lim = 1000,
-                 quiet = FALSE)
+.newrmst <- function(d)
 {
+    uD <- sort(unique.default(d))
+    n <- attr(d, "Size")
+
+    forest <- seq_len(n)
+    m <- matrix(NA_real_, n * (n - 1) / 2, 3L)
+    alt <- integer()
+    i <- 0L
+
+    for (x in uD) {
+        pairs <- sapply(which(d == x), getIandJ, n = n)
+        todrop <- forest[pairs[1L, ]] == forest[pairs[2L, ]]
+        if (all(todrop)) next
+        if (any(todrop)) pairs <- pairs[, !todrop, drop = FALSE]
+        Npairs <- ncol(pairs)
+        for (j in seq_len(Npairs)) {
+            h1 <- pairs[1L, j]
+            h2 <- pairs[2L, j]
+            k1 <- forest[h1]
+            k2 <- forest[h2]
+            if (k1 != k2) {
+                forest[forest == k2] <- k1
+                i <- i + 1L
+                m[i, ] <- c(h1, h2, x)
+            } else { # else a loop is created
+                i <- i + 1L
+                m[i, ] <- c(h1, h2, x)
+                alt <- c(alt, i)
+            }
+        }
+        if (length(unique(forest)) == 1L) break
+    }
+
+    m <- m[seq_len(i), , drop = FALSE]
+    colnames(m) <- c("", "", "step")
+    if (length(alt)) {
+        ALT <- m[alt, , drop = FALSE]
+        m <- m[-alt, , drop = FALSE]
+        attr(m, "alter.links") <- ALT
+    }
+    attr(m, "labels") <- attr(d, "Labels")
+    class(m) <- "haploNet"
+    m
+}
+
+rmst <- function(d, B = NULL, stop.criterion = NULL, iter.lim = 1000,
+                 quiet = FALSE, random = FALSE)
+{
+    if (!is.matrix(d) && !inherits(d, "dist"))
+        stop("'d' must be a matrix or a 'dist' object")
+
+    if (!inherits(d, "dist")) d <- as.dist(d)
+    n <- attr(d, "Size")
+
+    if (n < 3) {
+        if (!quiet)
+            warning("less than 3 haplotypes: returning the MST")
+        return(mst(d))
+    }
+
+    if (!random) return(.newrmst(d))
+
+    D <- as.matrix(d)
+
     updateMat <- function(mat, rep) {
         i <- match(mat, MAT)
         Nnew <- length(inew <- which(is.na(i)))
@@ -53,7 +115,8 @@ rmst <- function(d, B = NULL, stop.criterion = NULL, iter.lim = 1000,
     ## do MST and return vector of mode character with the labelled-links
     foo <- function(d) {
         d <<- as.dist(d)
-        rnt <<- rnt <- mst(d)
+        rnt <- mst(d)
+        assign("rnt", rnt, parent.env(environment()))
         m <- rnt[, 1:2]
         dm <- dim(m)
         mat <- attr(rnt, "labels")[m]
@@ -62,10 +125,6 @@ rmst <- function(d, B = NULL, stop.criterion = NULL, iter.lim = 1000,
         paste(mat[1, ], mat[2, ], sep = "\r")
     }
 
-    if (!is.matrix(d) && !inherits(d, "dist"))
-        stop("'d' must be a matrix or a 'dist' object")
-    D <- as.matrix(d)
-    n <- nrow(D)
     randomize <- TRUE
     if (n < 6 && is.null(B)) {
         B <- factorial(n)
@@ -353,7 +412,16 @@ haploNet <- function(h, d = NULL, getProb = TRUE)
         stop("'h' must be of class 'haplotype'")
     freq <- sapply(attr(h, "index"), length)
     n <- length(freq) # number of haplotypes
+    if (n == 0) return(NULL)
     link <- matrix(0, 0, 3)
+    if (n == 1) {
+        link <- cbind(link, NA)
+        colnames(link) <- c("", "", "step", "Prob")
+        attr(link, "freq") <- freq
+        attr(link, "labels") <- rownames(h)
+        class(link) <- "haploNet"
+        return(link)
+    }
     if (is.null(d)) {
         ## d <- dist.dna(h, "N", pairwise.deletion = TRUE)
         d <- .C(distDNA_pegas, h, as.integer(n), ncol(h), numeric(n * (n - 1)/2), NAOK = TRUE)[[4]]
@@ -411,12 +479,23 @@ print.haploNet <- function(x, ...)
 {
     cat("Haplotype network with:\n")
     cat(" ", length(attr(x, "labels")), "haplotypes\n")
-    N <- n <- nrow(x)
+    N <- nrow(x)
+    steps <- x[, 3]
     altlinks <- attr(x, "alter.links")
-    if (!is.null(altlinks)) N <- N + nrow(altlinks)
-    cat(" ", N, if (N > 1) "links\n" else "link\n")
-    cat("  link lengths between", x[1, 3], "and", x[n, 3], "step(s)\n\n")
-    cat("Use print.default() to display all elements.\n")
+    if (!is.null(altlinks)) {
+        steps <- c(steps, altlinks[, 3])
+        N <- length(steps)
+    }
+    cat(sprintf("  %d link%s\n", N, ifelse(N > 1, "s", "")))
+    if (N) {
+        if (N == 1) {
+            cat("  link length:", steps, "step(s)")
+        } else { # N > 1
+            rl <- range(steps)
+            cat("  link lengths between", rl[1], "and", rl[2], "step(s)")
+        }
+    }
+    cat("\n\nUse print.default() to display all elements.\n")
 }
 
 circle <- function(x, y, size, col = "black", pie = NULL, bg = NULL)
@@ -552,7 +631,9 @@ diamond <- function(x, y, size, col, pie = NULL, bg = NULL)
     ya0 <- yy[altlink[s, 1]]
     xa1 <- xx[altlink[s, 2]]
     ya1 <- yy[altlink[s, 2]]
-    segments(xa0, ya0, xa1, ya1, col = "grey", lty = 2)
+    OPTS <- get("plotHaploNetOptions", envir = .PlotHaploNetEnv)
+    segments(xa0, ya0, xa1, ya1, col = OPTS$link.color.alt,
+             lty = OPTS$link.type.alt, lwd = OPTS$link.width.alt)
     if (show.mutation) {
         n <- length(xx)
         .labelSegmentsHaploNet(xx, yy, altlink[s, 1:2, drop = FALSE],
@@ -679,7 +760,7 @@ diamond <- function(x, y, size, col, pie = NULL, bg = NULL)
         if (any(s <- deltaRadii != 0)) {
 ### if there are links between symbols of different size: need to place the
 ### annotation in the middle of the "visible" part of the link. We do that
-### with a weighting mean of the coordinates (simpler to calculate than
+### with a weighted mean of the coordinates (simpler to calculate than
 ### trogonometric formulas). We do that only for the different sized symbols.
             s <- which(s)
             ## lengths between each pair of nodes:
